#include "ADC.h"

volatile uint32_t adc_value;
volatile uint8_t frame_buffer[64];

void init_ADC()
{
    GPIOC->MODER &= ~(0x3 << 0);
    GPIOC->MODER |= (0x3 << 0);    // PC0 (ADC1_IN10) in analog mode
    
    RCC->APB2ENR |= (1 << 8) | (1 << 9) | (1 << 10);    // enable clock for ADC1, ADC2 and ADC3
    
    ADC1->SQR1 &= ~(0xF << 20);    // L = 0
    ADC1->SQR1 |= (0x1 << 20);     // L = 0
    ADC1->SQR3 &= ~(0x1F << 0);    // SQ1 = 0
    ADC1->SQR3 |= ((0x1F & 10) << 0);       // SQ1 = 10, so we use ADC1_IN10 (PC0)
    
    ADC1->SMPR2 |= (0x1 << 0);    // 15 cycles sample time (channel 0)
    ADC1->SMPR1 |= (0x1 << 0);    // 15 cycles sample time (channel 10)
    
    ADC1->CR2 |= (1 << 8);    // enable DMA
    ADC1->CR2 |= (1 << 9);    // enable DMA requests
    ADC1->CR2 |= (1 << 1);    // enable continous mode
    
    
    RCC->AHB1ENR |= (1 << 22);     // enable clock for DMA2 (which is used for ADC1, ADC2 and ADC3)
    NVIC->ISER[1] |= (1 << 28);    // enable interrupt 60 (DMA2_Stream4) (60 = 32 + 28)
    
    while(((DMA2->S4CR >> 0) & 0x1) == 1);
    
    DMA2->S4NDTR = 0xFFFF & 1;     // Transfer 1 data item in stream 4
    DMA2->S4PAR = (uint32_t)(&(ADC1->DR));     // origin address is ADC1's data register
    DMA2->S4M0AR = ((uint32_t)(&adc_value));     // destination address is the address of the variable adc_value
    
    DMA2->S4CR &= ~(0xF << 25);    // CHSEL = 0, to use channel 0 in stream 4 (ADC1)
    
    DMA2->HIFCR |= (1 << 5);       // CTCIF4 = 1 ==> reset TCIF4
    DMA2->S4CR |= (1 << 4);        // TCIE = 1 ==> transfer complete interrupt enabled
    DMA2->S4CR &= ~(0x3 << 11);
    DMA2->S4CR |= (0x2 << 11);     // peripheral data size = 32bit (word)
    
    DMA2->S4CR &= ~(0x3 << 13);
    DMA2->S4CR |= (0x2 << 13);     // memory size = 32bit (word)
    
    DMA2->S4CR |= (1 << 0);        // EN = 1 ==> stream 4 enabled
    
    
    ADC1->CR2 |= (1 << 0);    // enable ADC
    ADC1->CR2 |= (1 << 30);   // start conversion of regular channels
}

void DMA2_Stream4_IRQHandler()
{
    static uint32_t sample_counter = 0;
    static uint32_t local_index = 0;
    static uint32_t desc_idx = 0;
    
    if(((DMA2->HISR >> 5) & 0x1) == 1)    // TCIF4 = 1 ==> a transfer complete event happened on stream 4
    {
        DMA2->HIFCR |= (1 << 5);      // CTCIF4 = 1 ==> reset TCIF4
        DMA2->S4CR |= (1 << 0);       // EN = 1 ==> stream 4 enabled (why do we need to set this bit again in the interrupt handler? why and when is it reset?)
        
        if(N_requested_frames > 0)
        {
            //GPIOA->ODR |= (1 << 8);
            
            tx_buffer_ptr[(desc_idx*N_PACKET_BYTES) + 14 + 0 + 10*local_index] = (uint8_t)((sample_counter >> 24) & 0xFF);
            tx_buffer_ptr[(desc_idx*N_PACKET_BYTES) + 14 + 1 + 10*local_index] = (uint8_t)((sample_counter >> 16) & 0xFF);
            tx_buffer_ptr[(desc_idx*N_PACKET_BYTES) + 14 + 2 + 10*local_index] = (uint8_t)((sample_counter >> 8) & 0xFF);
            tx_buffer_ptr[(desc_idx*N_PACKET_BYTES) + 14 + 3 + 10*local_index] = (uint8_t)((sample_counter >> 0) & 0xFF);
            sample_counter++;
            
            // ADC1
            tx_buffer_ptr[(desc_idx*N_PACKET_BYTES) + 14 + 4 + 10*local_index] = (uint8_t)((adc_value >> 8) & 0xFF);
            tx_buffer_ptr[(desc_idx*N_PACKET_BYTES) + 14 + 5 + 10*local_index] = (uint8_t)((adc_value >> 0) & 0xFF);
            
            // TODO: ADC2
            tx_buffer_ptr[(desc_idx*N_PACKET_BYTES) + 14 + 6 + 10*local_index] = (uint8_t)((adc_value >> 8) & 0xFF);
            tx_buffer_ptr[(desc_idx*N_PACKET_BYTES) + 14 + 7 + 10*local_index] = (uint8_t)((adc_value >> 0) & 0xFF);
            
            // TODO: ADC3
            tx_buffer_ptr[(desc_idx*N_PACKET_BYTES) + 14 + 8 + 10*local_index] = (uint8_t)((adc_value >> 8) & 0xFF);
            tx_buffer_ptr[(desc_idx*N_PACKET_BYTES) + 14 + 9 + 10*local_index] = (uint8_t)((adc_value >> 0) & 0xFF);
            local_index++;
            
            // transmit an ethernet frame after 5 samples have been taken. N_PACKET_BYTES = 64 = (4 + (3*2))*5 + 14
            if(local_index == 5)
            {
                transmit_ethernet_frame(desc_idx);
                desc_idx = (desc_idx + 1) % 2;
                local_index = 0;
                N_requested_frames--;
            }
            
            //GPIOA->ODR &= ~(1 << 8);
        }
        else
        {
            sample_counter = 0;
        }
    }
}
