#include <stdint.h>

#ifndef STM32F100_TYPES_H
#define STM32F100_TYPES_H

typedef struct {
    uint32_t CR;
    uint32_t PLLCFGR;
    uint32_t CFGR;
    uint32_t CIR;
    uint32_t AHB1RSTR;
    uint32_t AHB2RSTR;
    uint32_t AHB3RSTR;
    uint32_t RESERVED0;
    uint32_t APB1RSTR;
    uint32_t APB2RSTR;
    uint32_t RESERVED1;
    uint32_t RESERVED2;
    uint32_t AHB1ENR;
    uint32_t AHB2ENR;
    uint32_t AHB3ENR;
    uint32_t RESERVED3;
    uint32_t APB1ENR;
    uint32_t APB2ENR;
    uint32_t RESERVED4;
    uint32_t RESERVED5;
    uint32_t AHB1LPENR;
    uint32_t AHB2LPENR;
    uint32_t AHB3LPENR;
    uint32_t RESERVED6;
    uint32_t APB1LPENR;
    uint32_t APB2LPENR;
    uint32_t RESERVED7;
    uint32_t RESERVED8;
    uint32_t BDCR;
    uint32_t CSR;
    uint32_t RESERVED9;
    uint32_t RESERVED10;
    uint32_t SSCGR;
    uint32_t PLLI2SCFGR;
    uint32_t PLLSAICFGR;
    uint32_t DCKFCGR1;
    uint32_t DCKCFGR2;
} RCC_T;

typedef struct {
    uint32_t MODER;
    uint32_t OTYPER;
    uint32_t OSPEEDR;
    uint32_t PUPDR;
    uint32_t IDR;
    uint32_t ODR;
    uint32_t BSRR;
    uint32_t LCKR;
    uint32_t AFRL;
    uint32_t AFRH;
} GPIO_T;

typedef struct {
    uint32_t ISER[8];
    uint32_t RESERVED0[24];
    uint32_t ICER[8];
    uint32_t ISPR[8];
    uint32_t RESERVED1[24];
    uint32_t ICPR[8];
    uint32_t IABR[8];
    uint32_t RESERVED3[56];
    uint32_t IPR[8];
} NVIC_T;

typedef struct {        // Offset
    uint32_t ACR;       // 0x00
    uint32_t KEYR;      // 0x04
    uint32_t OPTKEYR;   // 0x08
    uint32_t SR;        // 0x0C
    uint32_t CR;        // 0x10
    uint32_t OPTCR;     // 0x14
    uint32_t OPTCR1;    // 0x18
} FLASH_T;

typedef struct {
    uint32_t CR;
    uint32_t SWTRIGR;
    uint32_t DHR12R1;
    uint32_t DHR12L1;
    uint32_t DHR8R1;
    uint32_t DHR12R2;
    uint32_t DHR12L2;
    uint32_t DHR8R2;
    uint32_t DHR12RD;
    uint32_t DHR12LD;
    uint32_t DHR8DR;
    uint32_t DOR1;
    uint32_t DOR2;
    uint32_t SR;
} DAC_T;

typedef struct {
    uint32_t CR1;
    uint32_t CR2;
    uint32_t SMCR;
    uint32_t DIER;
    uint32_t SR;
    uint32_t EGR;
    uint32_t CCMR1;
    uint32_t CCMR2;
    uint32_t CCER;
    uint32_t CNT;
    uint32_t PSC;
    uint32_t ARR;
    uint32_t RESERVED0;
    uint32_t CCR1;
    uint32_t CCR2;
    uint32_t CCR3;
    uint32_t CCR4;
    uint32_t RESERVED1;
    uint32_t DCR;
    uint32_t DMAR;
    uint32_t OR;
} TIM_2_5_T;

typedef struct {
    uint32_t SR;
    uint32_t CR1;
    uint32_t CR2;
    uint32_t SMPR1;
    uint32_t SMPR2;
    uint32_t JOFR1;
    uint32_t JOFR2;
    uint32_t JOFR3;
    uint32_t JOFR4;
    uint32_t HTR;
    uint32_t LTR;
    uint32_t SQR1;
    uint32_t SQR2;
    uint32_t SQR3;
    uint32_t JSQR;
    uint32_t JDR1;
    uint32_t JDR2;
    uint32_t JDR3;
    uint32_t JDR4;
    uint32_t DR;
} ADC_T;

typedef struct {
    uint32_t CSR;
    uint32_t CCR;
    uint32_t CDR;
} ADC_COMMON_T;

typedef struct {
    uint32_t LISR;
    uint32_t HISR;
    uint32_t LIFCR;
    uint32_t HIFCR;
    uint32_t S0CR;
    uint32_t S0NDTR;
    uint32_t S0PAR;
    uint32_t S0M0AR;
    uint32_t S0M1AR;
    uint32_t S0FCR;
    uint32_t S1CR;
    uint32_t S1NDTR;
    uint32_t S1PAR;
    uint32_t S1M0AR;
    uint32_t S1M1AR;
    uint32_t S1FCR;
    uint32_t S2CR;
    uint32_t S2NDTR;
    uint32_t S2PAR;
    uint32_t S2M0AR;
    uint32_t S2M1AR;
    uint32_t S2FCR;
    uint32_t S3CR;
    uint32_t S3NDTR;
    uint32_t S3PAR;
    uint32_t S3M0AR;
    uint32_t S3M1AR;
    uint32_t S3FCR;
    uint32_t S4CR;
    uint32_t S4NDTR;
    uint32_t S4PAR;
    uint32_t S4M0AR;
    uint32_t S4M1AR;
    uint32_t S4FCR;
    uint32_t S5CR;
    uint32_t S5NDTR;
    uint32_t S5PAR;
    uint32_t S5M0AR;
    uint32_t S5M1AR;
    uint32_t S5FCR;
    uint32_t S6CR;
    uint32_t S6NDTR;
    uint32_t S6PAR;
    uint32_t S6M0AR;
    uint32_t S6M1AR;
    uint32_t S6FCR;
    uint32_t S7CR;
    uint32_t S7NDTR;
    uint32_t S7PAR;
    uint32_t S7M0AR;
    uint32_t S7M1AR;
    uint32_t S7FCR;
} DMA_T;

typedef struct {
    uint32_t CR1;
    uint32_t CR2;
    uint32_t CR3;
    uint32_t BRR;
    uint32_t GTPR;
    uint32_t RTOR;
    uint32_t RQR;
    uint32_t ISR;
    uint32_t ICR;
    uint32_t RDR;
    uint32_t TDR;
} USART_T;

typedef struct {
    uint32_t MACCR;          // 0x00
    uint32_t MACFFR;         // 0x04
    uint32_t MACHTHR;        // 0x08
    uint32_t MACHTLR;        // 0x0C
    uint32_t MACMIIAR;       // 0x10
    uint32_t MACMIIDR;       // 0x14
    uint32_t MACFCR;         // 0x18
    uint32_t MACVLANTR;      // 0x1C
    uint32_t RESERVED0[2];   // 0x20..0x24 ok
    uint32_t MACRWUFFR;      // 0x28
    uint32_t MACPMTCSR;      // 0x2C
    uint32_t RESERVED1;      // 0x30
    uint32_t MACDBGR;        // 0x34
    uint32_t MACSR;          // 0x38
    uint32_t MACIMR;         // 0x3C
    uint32_t MACA0HR;        // 0x40
    uint32_t MACA0LR;        // 0x44
    uint32_t MACA1HR;        // 0x48
    uint32_t MACA1LR;        // 0x4C
    uint32_t MACA2HR;        // 0x50
    uint32_t MACA2LR;        // 0x54
    uint32_t MACA3HR;        // 0x58
    uint32_t MACA3LR;        // 0x5C
    uint32_t RESERVED2[40];  // 0x60..0xFC ok
    uint32_t MMCCR;          // 0x100
    uint32_t MMCRIR;         // 0x104
    uint32_t MMCTIR;         // 0x108
    uint32_t MMCRIMR;        // 0x10C
    uint32_t MMCTIMR;        // 0x110
    uint32_t RESERVED3[14];  // 0x114..0x148 ok
    uint32_t MMCTGFSCCR;     // 0x14C
    uint32_t MMCTGRMSCCR;    // 0x150
    uint32_t RESERVED4[5];   // 0x154..0x164 ok
    uint32_t MMCTGFCR;       // 0x168
    uint32_t RESERVED5[10];  // 0x16C..0x190 ok
    uint32_t MMCRFCECR;      // 0x194
    uint32_t MMCRFAECR;      // 0x198
    uint32_t RESERVED6[10];  // 0x19C..0x1C0 ok
    uint32_t MMCRGUFCR;      // 0x1C4
    uint32_t RESERVED7[334]; // 0x1C8..0x6FC ok
    uint32_t PTPTSCR;        // 0x700
    uint32_t PTPSSIR;        // 0x704
    uint32_t PTPTSHR;        // 0x708
    uint32_t PTPTSLR;        // 0x70C
    uint32_t PTPTSHUR;       // 0x710
    uint32_t PTPTSLUR;       // 0x714
    uint32_t PTPTSAR;        // 0x718
    uint32_t PTPTTHR;        // 0x71C
    uint32_t PTPTTLR;        // 0x720
    uint32_t RESERVED8;      // 0x724
    uint32_t PTPTSSR;        // 0x728
    uint32_t RESERVED9[565]; // 0x72C..0x0FFC ok
    uint32_t DMABMR;         // 0x1000
    uint32_t DMATPDR;        // 0x1004
    uint32_t DMARPDR;        // 0x1008
    uint32_t DMARDLAR;       // 0x100C
    uint32_t DMATDLAR;       // 0x1010
    uint32_t DMASR;          // 0x1014
    uint32_t DMAOMR;         // 0x1018
    uint32_t DMAIER;         // 0x101C
    uint32_t DMAMFBOCR;      // 0x1020
    uint32_t DMARSWTR;       // 0x1024
    uint32_t RESERVED10[8];  // 0x1028..0x1044 ok
    uint32_t DMACHTDR;       // 0x1048
    uint32_t DMACHRDR;       // 0x104C
    uint32_t DMACHTBAR;      // 0x1050
    uint32_t DMACHRBAR;      // 0x1054
} ETH_T;

typedef struct {
    uint32_t MEMRMP;
    uint32_t PMC;
    uint32_t EXTICR1;
    uint32_t EXTICR2;
    uint32_t EXTICR3;
    uint32_t EXTICR4;
    uint32_t CBR;
    uint32_t CMPCR;
} SYSCFG_T;

#endif
